@@ -1,48 +1,56 @@
 #' @import ggplot2
 #' @import reshape2
 
-fut_to_zero <- function(fut) {
-  (cumprod(1+fut))^(1/(seq_along(fut)))-1
+fut_to_zero_eff <- function(fut) {
+  (cumprod(1 + fut)) ^ (1 / (seq_along(fut))) - 1
 }
 
 disc_to_swap <- function(disc) {
-  (1-disc) / cumsum(disc)
+  (1 - disc) / cumsum(disc)
 }
 
 swap_to_disc <- function(swap) {
   d = swap
   d[1] = 1 / (1 + swap[1])
   for (j in 2:length(swap)) {
-    d[j] = (1 - sum(d[1:(j-1)])*swap[j]) / (1 + swap[j])
+    d[j] = max((1 - sum(d[1:(j - 1)])*swap[j]) / (1 + swap[j]), 0)
   }
   d
 }
 
-disc_to_zero <- function(disc) {
-  (1 / disc)^(1/(seq_along(disc)))-1
+disc_to_zero_eff <- function(disc) {
+  (1 / disc) ^ (1 / (seq_along(disc))) - 1
 }
 
-zero_to_disc <- function(zero) {
-  1 / ( (1 + zero)^(seq_along(zero)) )
+zero_eff_to_disc <- function(zero) {
+  1 / ((1 + zero) ^ (seq_along(zero)) )
+}
+
+disc_to_zero_nom <- function(disc) {
+  (1 / disc - 1) / seq_along(disc)
+}
+
+zero_nom_to_disc <- function(zero) {
+  1 / (1 + zero * seq_along(zero))
 }
 
 disc_to_fut <- function(disc) {  
-  exp(-diff(log(c(1,disc))))-1  
+  exp(-diff(log(c(1,disc)))) - 1  
 }
 
 fut_to_disc <- function(fut) {
-  1 / cumprod(1+fut)
+  1 / cumprod(1 + fut)
 }
 
 disc_to_german <- function(disc) {
   vapply(
     1:length(disc),
-    function(i) ( 1 - 1 / i * sum(disc[1:i]) ) / sum( disc[1:i] * (1 - ( 1:i - 1 ) / i ) ),
+    function(i) (1 - 1 / i * sum(disc[1:i]) ) / sum( disc[1:i] * (1 - (1:i - 1 ) / i ) ),
     1)  
 }
 
 disc_to_french <- function(disc, search_interval = c(0.0001,1), tol = 1e-8) {  
-  zerome = function(r,i,disc) 1/r * ( 1 - (1+r)^(-i) ) - sum(disc[1:i])
+  zerome = function(r,i,disc) 1/r * (1 - (1 + r) ^ (-i) ) - sum(disc[1:i])
   vapply(
     1:length(disc),
     function(i) uniroot(function(r) zerome(r,i,disc), interval = search_interval, tol = tol)$root,
@@ -60,30 +68,32 @@ dir_to_eff <- function(r) {
 #' @title Creates a rate curve instance
 #' 
 #' @param rates A rate vector
-#' @param rate_type The rate type. Must be on of c("fut","zero","swap")
+#' @param rate_type The rate type. Must be on of c("fut", "zero_nom", "zero_eff", "swap")
 #' @param pers The periods the rates correspond to
 #' @param fun_d A discount factor function. fun_d(x) returns the discount factor for time x, vectorized on x
 #' @param fun_r A rate function. fun_r(x) returns the EPR for time x, vectorized on x
-#' @param knots The nodes used to bootstrap the rates
+#' @param knots The nodes used to bootstrap the rates. This is a mandatory argument if a rate function or discount function is provided
+#' @param functor A function with parameters x and y, that returns a function used to interpolate
 #' 
 #' @note Currently a rate curve can only be built from one of the following sources
 #' \enumerate{
 #' \item A discount factor function
-#' \item A rate function and a rate type from the following types: "fut", "zero" or "swap"
+#' \item A rate function and a rate type from the following types: "fut", "zero_nom", "zero_eff" or "swap"
 #' \item A rate vector, a pers vector and a rate type as before
 #' }
 #' @examples
-#' rate_curve(rates = c(0.1, 0.2, 0.3), rate_type = "zero")
-#' rate_curve(fun_r = function(x) rep_len(0.1, length(x)), rate_type = "swap")
-#' rate_curve(fun_d = function(x) 1 / (1 + x))
+#' rate_curve(rates = c(0.1, 0.2, 0.3), rate_type = "zero_eff")
+#' rate_curve(fun_r = function(x) rep_len(0.1, length(x)), rate_type = "swap", knots = 1:12)
+#' rate_curve(fun_d = function(x) 1 / (1 + x), knots = 1:12)
 #' @export
 rate_curve <- function(
   rates = NULL,
-  rate_type = "zero",
+  rate_type = "zero_eff",
   pers = 1:length(rates),
   fun_d = NULL,
   fun_r = NULL,
-  knots = 1:length(rates)) {
+  knots = seq.int(from = 1, to = max(pers), by = 1),
+  functor = function(x, y) splinefun(x = x, y = y, method = "monoH.FC")) {
   if (!(
     (!is.null(fun_d) &&  !is.null(knots)) ||
       (!is.null(fun_r) && !is.null(rate_type) && !is.null(knots)) ||
@@ -94,25 +104,25 @@ rate_curve <- function(
     r <- structure(list(), class = "rate_curve")
     r$f <- fun_d
     r$knots <- knots
+    r$functor <- functor
     r
   } else if (!is.null(fun_r)) {
     d <- do.call(what = paste0(rate_type,"_to_disc"), args = list(fun_r(knots)))
-    log_f <- approxfun(x = knots, y = log(d), method = "linear", rule = 2)
-    f <- function(x) exp(log_f(x))
-    rate_curve(fun_d = f, knots = knots)
+    f <- functor(x = knots, y = d)    
+    rate_curve(fun_d = f, knots = knots, functor = functor)
   } else if (!is.null(rates)) {
-    f <- approxfun(x = pers, y = rates, method = "linear", rule = 2)
-    rate_curve(fun_r = f, rate_type = rate_type, knots = knots)
+    f <- functor(x = pers, y = rates)
+    rate_curve(fun_r = f, rate_type = rate_type, knots = knots, functor = functor)
   } else {
     stop("The rate_curve constructor lacks arguments")
   }  
 }
 
-get_rate_fun <- function(r, rate_type = "zero") {
-  stopifnot(rate_type %in% c("french","fut","german","zero","swap"))
+get_rate_fun <- function(r, rate_type = "zero_eff") {
+  stopifnot(rate_type %in% c("french","fut","german","zero_eff","zero_nom","swap"))
   d <- (r$f)(r$knots)
   y <- do.call(what = paste0("disc_to_",rate_type), args = list(d))
-  approxfun(x = r$knots, y = y, method = "linear", rule = 2)
+  r$functor(x = r$knots, y = y)
 }
 
 #' @title Returns a particular rate or rates from a curve
@@ -124,13 +134,13 @@ get_rate_fun <- function(r, rate_type = "zero") {
 #' @return If \code{x} is \code{NULL}, then returns a rate function of \code{rate_type} type.
 #' Else, it returns the rates of \code{rate_type} type and corresponding to time \code{x}
 #' @examples
-#' r <- rate_curve(rates = c(0.1, 0.2, 0.3), rate_type = "zero")
-#' r["zero"]
+#' r <- rate_curve(rates = c(0.1, 0.2, 0.3), rate_type = "zero_eff")
+#' r["zero_eff"]
 #' r["swap",c(1.5, 2)]
 #' @export
-`[.rate_curve` <- function(r, rate_type = "zero", x = NULL) {
+`[.rate_curve` <- function(r, rate_type = "zero_eff", x = NULL) {
   f <- get_rate_fun(r = r, rate_type = rate_type)
-  if(is.null(x))
+  if (is.null(x))
     f
   else
     f(x)
@@ -139,10 +149,10 @@ get_rate_fun <- function(r, rate_type = "zero") {
 #' @title Plots a rate curve
 #' 
 #' @param x The rate curve
-#' @param rate_type The rate types to plot, in c("french","fut","german","zero","swap")
+#' @param rate_type The rate types to plot, in c("french","fut","german","zero_eff","zero_nom","swap")
 #' @param ... Other arguments (unused)
 #' @examples
-#' r <- rate_curve(rates = c(0.1, 0.2, 0.3), rate_type = "zero")
+#' r <- rate_curve(rates = c(0.1, 0.2, 0.3), rate_type = "zero_eff")
 #' plot(r)
 #' \dontrun{
 #' plot(r, rate_type = "german")
@@ -150,7 +160,7 @@ get_rate_fun <- function(r, rate_type = "zero") {
 #' }
 #' @export
 plot.rate_curve <- function(x, rate_type = NULL, ...) {
-  all_rate_types = c("french","fut","german","zero","swap")
+  all_rate_types = c("french","fut","german","zero_eff","zero_nom","swap")
   if (is.null(rate_type)) {
     rate_type = all_rate_types
   }
